% Competitive-adaptive DE algorithm for boundary constrained problems
% ===================================================================
% 
% The shift of the global minimum point is enabled
% set input parameter shift=[0,0,...,0] if you do not want to shift
%
% Tvrdik, Feb  2008, modified Sept. 2014
%
% INPUT PARAMETERS:
% f_near ... value of function for which numeber of function evaluation 
%            is also reported in func_near
% shift  ... set input parameter shift=[0,0,...,0] if you do not want to
%            shift
% the other input parameters agree with those described in the website
%
% OUTPUT PARAMETERS:
% success ... number of cases if trial point y is good 
%             for renewing population 
% nrst    ... number of probaility' resets  to initial values
% cni     ... frequencies of successful exploitation of 
%             different strategies (vector)
% meaning of the remaining parameters is obvious (selfexplaining)
%
function [x_star,fn_star, func_evals, success, func_near, nrst, cni]=...
    b6e6rl(fn_name, a, b, N, my_eps, max_evals, f_near, n0, delta, shift)
%
h=12;
% initialization
d=length(a); cni=zeros(1,h);  ni=zeros(1,h)+n0;  nrst=0;
func_evals=N;  success=0; func_near=0;
P=zeros(N,d+1);
for i=1:N
    P(i,1:d)=a+(b-a).*rand(1,d);
    P(i,d+1)=feval(fn_name,P(i,1:d)-shift);
end  % 0-th generation initialized
fmax=max(P(:,d+1));
[fmin, indmin]=min(P(:,d+1));
Q=P;
p2=0.5*(1+1/d);
p1=0.5*(p2+1/d);
p3=0.5*(p2+1);
CR1=CRexp_set(p1,d);
CR2=CRexp_set(p2,d);
CR3=CRexp_set(p3,d);
while (fmax-fmin > my_eps) && (func_evals < d*max_evals)  % main loop
    for i=1:N  %  in each generation
        [hh p_min]=roulete(ni); % select hh-th set of parameters
        if p_min<delta
            cni=cni+ni-n0;
            ni=zeros(1,h)+n0;
            nrst=nrst+1;
        end  %reset
        switch hh % number of selected heuristic
            case 1
                F=0.5; CR=0; y=derand_RL(P,F,CR,i);
            case 2
                F=0.5; CR=0.5; y=derand_RL(P,F,CR,i);
            case 3
                F=0.5; CR=1; y=derand_RL(P,F,CR,i);
            case 4
                F=0.8; CR=0; y=derand_RL(P,F,CR,i);
            case 5
                F=0.8; CR=0.5; y=derand_RL(P,F,CR,i);
            case 6
                F=0.8; CR=1; y=derand_RL(P,F,CR,i);
            case 7
                F=0.5; CR=CR1;    y=derandexp_RL(P,F,CR,i);
            case 8
                F=0.5; CR=CR2;    y=derandexp_RL(P,F,CR,i);
            case 9
                F=0.5; CR=CR3;   y=derandexp_RL(P,F,CR,i);
            case 10
                F=0.8; CR=CR1;   y=derandexp_RL(P,F,CR,i);
            case 11
                F=0.8; CR=CR2;   y=derandexp_RL(P,F,CR,i);
            case 12
                F=0.8; CR=CR3;   y=derandexp_RL(P,F,CR,i);
        end
        y=zrcad(y,a,b);     % perturbation
        fy=feval(fn_name,y-shift);
        func_evals=func_evals+1;
        if fy < P(i,d+1)  % trial point y is good for renewing population
            Q(i,:)= [y fy];
            success=success+1;
            ni(hh)=ni(hh)+1;         % change of qi
        end
    end % end of generation
    P=Q;
    fmax=max(P(:,d+1));
    [fmin,indmin]=min(P(:,d+1));
    if func_near==0 && fmin<f_near
        func_near=func_evals;
    end
end   % main loop - end
cni=cni+ni-n0;
x_star=P(indmin,1:d);
fn_star=fmin;
%